@@ -39,9 +39,10 @@ class UNetPolicy(DSPolicy):
     def match(self, module):
         return isinstance(module, self._orig_layer_class)
 
-    def apply(self, module):
-        from .unet import DSUNet
-        return DSUNet(module)
+    def apply(self, module, enable_cuda_graph=True):
+        # TODO(cmikeh2): Enable cuda graph should be an inference configuration
+        from ..model_implementations.diffusers.unet import DSUNet
+        return DSUNet(module, enable_cuda_graph=enable_cuda_graph)
 
     def attention(self, client_module):
         qw = client_module.to_q.weight
@@ -67,6 +68,24 @@ class UNetPolicy(DSPolicy):
                    client_module.heads
 
 
+class VAEPolicy(DSPolicy):
+    def __init__(self):
+        super().__init__()
+        try:
+            import diffusers
+            self._orig_layer_class = diffusers.models.vae.AutoencoderKL
+        except ImportError:
+            self._orig_layer_class = None
+
+    def match(self, module):
+        return isinstance(module, self._orig_layer_class)
+
+    def apply(self, module, enable_cuda_graph=True):
+        # TODO(cmikeh2): Enable cuda graph should be an inference configuration
+        from ..model_implementations.diffusers.vae import DSVAE
+        return DSVAE(module, enable_cuda_graph=enable_cuda_graph)
+
+
 class TransformerPolicy(DSPolicy):
     # a static class variable containing the HuggingFace model configuration.
     # see e.g., transformers.models.opt.configuration_opt.OPTConfig
@@ -185,7 +204,7 @@ class HFBertLayerPolicy(TransformerPolicy):
 
 class HFCLIPLayerPolicy(TransformerPolicy):
     def __init__(self, client_module, inference=False):
-        super().__init__(inference, pre_attn_norm=True, scale_attention=False)
+        super().__init__(inference, pre_attn_norm=True, scale_attention=True)
         self.client_module = client_module
         self.cuda_graph_supported = True
 
@@ -608,4 +627,4 @@ replace_policies = [
 ]
 
 # non-transformer-based policies
-generic_policies = [UNetPolicy]
+generic_policies = [UNetPolicy, VAEPolicy]
