@@ -3,23 +3,19 @@ Copyright 2020 The Microsoft DeepSpeed Team
 '''
 import json
 import math
-import importlib
 import torch
-from torch import nn
 from torch.autograd import Function
-import time
 from ... import op_builder
 #from ...inference.engine import inference_cuda_module, specialized_mode
 # Cuda modules will be imported if needed
 inference_cuda_module = None
 specialized_mode = None
 import torch.nn as nn
-from .transformer_inference import DeepSpeedSelfAttention, DeepSpeedInferenceConfig
+from .ds_attention import DeepSpeedSelfAttention
+from .config import DeepSpeedInferenceConfig
 from ....moe.sharded_moe import TopKGate
 from deepspeed import comm as dist
 
-import torch.nn.functional as F
-
 
 class DeepSpeedMoEInferenceConfig(DeepSpeedInferenceConfig):
     """Initialize the DeepSpeed Transformer Config.
@@ -72,7 +68,8 @@ class DeepSpeedMoEInferenceConfig(DeepSpeedInferenceConfig):
                  noisy_gate_policy=None,
                  drop_tokens=True,
                  use_rts=False,
-                 mlp_type='standard'):
+                 mlp_type='standard',
+                 scale_attn_by_inverse_layer_idx=False):
         super(DeepSpeedMoEInferenceConfig,
               self).__init__(
                   hidden_size,
@@ -101,6 +98,7 @@ class DeepSpeedMoEInferenceConfig(DeepSpeedInferenceConfig):
         self.use_rts = use_rts
         self.global_experts = global_experts
         self.mlp_type = mlp_type
+        self.scale_attn_by_inverse_layer_idx = scale_attn_by_inverse_layer_idx
 
     @classmethod
     def from_dict(cls, json_object):
