@@ -41,7 +41,8 @@ class DeepSpeedDataLoader(object):
                  num_local_io_workers=None,
                  data_sampler=None,
                  data_parallel_world_size=None,
-                 data_parallel_rank=None):
+                 data_parallel_rank=None,
+                 dataloader_drop_last=False):
         self.tput_timer = tput_timer
         self.batch_size = batch_size
 
@@ -67,8 +68,14 @@ class DeepSpeedDataLoader(object):
         self.device_count = device_count
         self.batch_size = batch_size
         self.pin_memory = pin_memory
-        self.len = len(self.data_sampler)
         self.data = None
+        self.dataloader_drop_last = dataloader_drop_last
+
+        if self.dataloader_drop_last:
+            self.len = len(self.data_sampler) // self.batch_size
+        else:
+            from math import ceil
+            self.len = ceil(len(self.data_sampler) / self.batch_size)
 
     def __iter__(self):
         self._create_dataloader()
@@ -88,14 +95,16 @@ class DeepSpeedDataLoader(object):
                                          batch_size=self.batch_size,
                                          pin_memory=self.pin_memory,
                                          sampler=self.data_sampler,
-                                         num_workers=self.num_local_io_workers)
+                                         num_workers=self.num_local_io_workers,
+                                         drop_last=self.dataloader_drop_last)
         else:
             self.dataloader = DataLoader(self.dataset,
                                          batch_size=self.batch_size,
                                          pin_memory=self.pin_memory,
                                          sampler=self.data_sampler,
                                          collate_fn=self.collate_fn,
-                                         num_workers=self.num_local_io_workers)
+                                         num_workers=self.num_local_io_workers,
+                                         drop_last=self.dataloader_drop_last)
         self.data = (x for x in self.dataloader)
 
         return self.dataloader
